/*
 * =============================================================================
 * 
 *   Copyright (c) 2011, The THYMELEAF team (http://www.thymeleaf.org)
 * 
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * 
 * =============================================================================
 */
package org.thymeleaf.spring3;

import java.util.Collections;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.MessageSource;
import org.springframework.context.MessageSourceAware;
import org.thymeleaf.spring3.dialect.SpringThymeleafDialect;
import org.thymeleaf.spring3.messageresolver.SpringMessageResolver;
import org.thymeleaf.TemplateEngine;
import org.thymeleaf.dialect.IDialect;
import org.thymeleaf.exceptions.ConfigurationException;




/**
 * 
 * @author Daniel Fern&aacute;ndez
 * 
 * @since 1.0
 *
 */
public class SpringThymeleafTemplateEngine 
        extends TemplateEngine 
        implements MessageSourceAware, InitializingBean {

    
    private static final SpringThymeleafDialect SPRING_THYMELEAF_DIALECT = new SpringThymeleafDialect();
    
    private MessageSource messageSource;
    
    
    public SpringThymeleafTemplateEngine() {
        super();
        super.setDialect(SPRING_THYMELEAF_DIALECT);
    }



    public void setMessageSource(final MessageSource messageSource) {
        this.messageSource = messageSource;
    }
    

    public void afterPropertiesSet() throws Exception {
        final SpringMessageResolver springMessageResolver = new SpringMessageResolver();
        springMessageResolver.setMessageSource(this.messageSource);
        super.setDefaultMessageResolvers(Collections.singleton(springMessageResolver));
    }
    

    @Override
    public void setDialect(final IDialect dialect) {
        throw new ConfigurationException(
                "Cannot set the dialect. " + SpringThymeleafTemplateEngine.class.getName() + 
                " will always use " + SpringThymeleafDialect.class.getName() + " as a dialect, " +
        		"and that cannot be changed. If you need a different dialect, consider using a " + 
                TemplateEngine.class.getName() + " instance instead.");
    }




    
    
    
}
