/*
 * =============================================================================
 * 
 *   Copyright (c) 2011, The THYMELEAF team (http://www.thymeleaf.org)
 * 
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * 
 * =============================================================================
 */
package org.thymeleaf.spring3.context;

import java.util.LinkedHashMap;
import java.util.Locale;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.context.ApplicationContext;
import org.thymeleaf.context.WebContext;

/**
 * 
 * @author Daniel Fern&aacute;ndez
 * 
 * @since 1.0
 *
 */
public class SpringWebContext 
        extends WebContext {
    

    public static final String BEANS_VARIABLE_NAME = "beans";
    

    
    public SpringWebContext(final HttpServletRequest request, final Locale locale, final ApplicationContext appctx) {
        this(request, locale, new LinkedHashMap<String, Object>(), appctx);
    }
    

    public SpringWebContext(final HttpServletRequest request, final Locale locale, final Map<String, ?> variables, final ApplicationContext appctx) {
        super(request, locale, addBeansVariable(variables, appctx));
    }
    

    private static Map<String,Object> addBeansVariable(final Map<String, ?> variables, final ApplicationContext appctx) {

        final Map<String,Object> newVariables =
            (variables == null?
                    new LinkedHashMap<String, Object>() : new LinkedHashMap<String, Object>(variables));
        
        final Beans beans = new Beans(appctx);
        
        newVariables.put(BEANS_VARIABLE_NAME, beans);
        
        return newVariables;
        
    }
    
}
