/*
 * =============================================================================
 * 
 *   Copyright (c) 2011, The THYMELEAF team (http://www.thymeleaf.org)
 * 
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 * 
 *       http://www.apache.org/licenses/LICENSE-2.0
 * 
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * 
 * =============================================================================
 */
package org.thymeleaf.spring3.expression.spel;

import java.util.LinkedHashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.expression.spel.standard.SpelExpression;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.thymeleaf.Arguments;
import org.thymeleaf.TemplateEngine;
import org.thymeleaf.expression.AbstractExpressionEvaluator;
import org.thymeleaf.spring3.expression.Fields;
import org.thymeleaf.templateresolver.TemplateResolution;

/**
 * 
 * @author Daniel Fern&aacute;ndez
 * 
 * @since 1.0
 *
 */
public final class SpelExpressionEvaluator 
        extends AbstractExpressionEvaluator {

    
    public static final String FIELDS_EVALUATION_VARIABLE_NAME = "fields";
    
    
    public static final SpelExpressionEvaluator INSTANCE = new SpelExpressionEvaluator();
    
    private static final Logger logger = LoggerFactory.getLogger(SpelExpressionEvaluator.class);

    private static final SpelExpressionParser PARSER = new SpelExpressionParser();
    


    
    public SpelExpressionEvaluator() {
        super();
    }
    
    
    public Object evaluate(
            final Arguments arguments, final TemplateResolution templateResolution, 
            final String spelExpression, final Object root, 
            final Map<String,Object> additionalContextVariables) {
        
        if (logger.isTraceEnabled()) {
            logger.trace("[THYMELEAF][{}] SpringEL expression: evaluating expression \"{}\" on target", TemplateEngine.threadIndex(), spelExpression);
        }

        final Map<String,Object> contextVariables = 
            new LinkedHashMap<String,Object>(getDefaultEvaluationVariables(arguments, templateResolution));
        contextVariables.putAll(additionalContextVariables);

        final StandardEvaluationContext context = new StandardEvaluationContext();
        context.addPropertyAccessor(VariablesMapPropertyAccessor.INSTANCE);
        context.addPropertyAccessor(BeansPropertyAccessor.INSTANCE);
        context.setVariables(contextVariables);
        
        final SpelExpression exp = (SpelExpression) PARSER.parseExpression(spelExpression);
        return exp.getValue(context, root);

    }


    
    
    @Override
    protected Map<String, Object> getDefaultEvaluationVariables(
            Arguments arguments, TemplateResolution templateResolution) {
        
        final Map<String,Object> variables = 
            super.getDefaultEvaluationVariables(arguments, templateResolution);
        
        final Fields forms = new Fields(arguments, templateResolution);
        variables.put(FIELDS_EVALUATION_VARIABLE_NAME, forms);
        
        return variables;
        
    }

    
    
}
